@@ -153,6 +153,10 @@ type context_type =
 
 external create_context : protocol -> context_type -> context = "ocaml_ssl_create_context"
 
+external add_extra_chain_cert : context -> string -> unit = "ocaml_ssl_ctx_add_extra_chain_cert"
+
+external add_cert_to_store : context -> string -> unit = "ocaml_ssl_ctx_add_cert_to_store"
+
 external use_certificate : context -> string -> string -> unit = "ocaml_ssl_ctx_use_certificate"
 
 external use_certificate_from_string : context -> string -> string -> unit = "ocaml_ssl_ctx_use_certificate_from_string"
@@ -179,6 +183,8 @@ external get_verify_result : socket -> int = "ocaml_ssl_get_verify_result"
 
 external get_verify_error_string : int -> string = "ocaml_ssl_get_verify_error_string"
 
+external digest : [`SHA1 | `SHA256 | `SHA384] -> certificate -> string = "ocaml_ssl_digest"
+
 type verify_mode =
   | Verify_peer
   | Verify_fail_if_no_peer_cert
@@ -253,6 +259,8 @@ external set_hostflags : socket -> x509_check_flag list -> unit = "ocaml_ssl_set
 
 external set_host : socket -> string -> unit = "ocaml_ssl_set1_host"
 
+external set_ip : socket -> string -> unit = "ocaml_ssl_set1_ip"
+
 external write : socket -> Bytes.t -> int -> int -> int = "ocaml_ssl_write"
 
 external write_substring : socket -> string -> int -> int -> int = "ocaml_ssl_write"
@@ -274,14 +282,10 @@ external accept : socket -> unit = "ocaml_ssl_accept"
 
 external flush : socket -> unit = "ocaml_ssl_flush"
 
-external shutdown : socket -> unit = "ocaml_ssl_shutdown"
+external ssl_shutdown : socket -> bool = "ocaml_ssl_shutdown"
 
 let open_connection_with_context context sockaddr =
-  let domain =
-    match sockaddr with
-      | Unix.ADDR_UNIX _ -> Unix.PF_UNIX
-      | Unix.ADDR_INET(_, _) -> Unix.PF_INET
-  in
+  let domain = Unix.domain_of_sockaddr sockaddr in
   let sock =
     Unix.socket domain Unix.SOCK_STREAM 0 in
     try
@@ -294,6 +298,12 @@ let open_connection_with_context context sockaddr =
 let open_connection ssl_method sockaddr =
   open_connection_with_context (create_context ssl_method Client_context) sockaddr
 
+let close_notify = ssl_shutdown
+
+let shutdown sock =
+  if not (close_notify sock)
+  then ignore (close_notify sock : bool)
+
 let shutdown_connection = shutdown
 
 let output_string ssl s =
